/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

#include "MimicParser.h"
#include "ParserRegistration.h"

int MimicParser::foo = 0;

typedef UtilPtr<Recorder> RecorderPtr;
typedef UtilPtr<ScorePlayer> ScorePlayerPtr;
typedef UtilPtr<Score> ScorePtr;

static RecorderPtr r;
static int note_count;
static ScheduleTime t;
static ioref node;
static ScorePlayerPtr p;

static void do_note(MIDI_msg m)
{
	m.node = node;
	r->insert(&m);
	oap->write(&m);
}

static void note_off(MIDI_msg* m)
{
	note_count--;
	t = oap->time();
	do_note(*m);
}

static void note_on(MIDI_msg* m)
{
	if (m->vel() == 0)
		note_off(m);
	else
	{
		note_count++;
		do_note(*m);
	}
}

MimicParser::MimicParser()
{
	note_count = 0;
	msg_hnd[MIDI_msg::note_on] = note_on;
	msg_hnd[MIDI_msg::note_off] = note_off;
	r = new Recorder;
	StartIdling();
	oap->open_input("some_input");
	node = oap->output_refnum("some_output");
	::t = oap->time();
}

MimicParser::~MimicParser()
{
	StopIdling();
	r = NULL; // free any Recorder (since it's static)
}

void MimicParser::SpendTime(const EventRecord&)
{
	ScheduleTime now = oap->time();
	if (p != NULL && p->isPlaying())
		::t = now;
	if ((note_count == 0) && ((now - ::t) > 4000)) // silence for 2 seconds?
	{
		ScorePtr s = new Score(r);
		p = new ScorePlayer(s);
		p->start();
		r = new Recorder(); // empty the Recorder
		::t = now;
	}
}

static parser* newFunc()
{
	return (new MimicParser());
}

static ParserRegistration reg(newFunc, "MimicParser");

