/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

#include "sched.h"
#include "io.h"
#include <process.h>

void
task_pool::
init( )
{
}

task*
task_pool::
store( const task_data& td )
{
	task* t = allocate(); if (!t) return NULL;
	t->td = td;
	mon.inc();
	return t;
}


task_pool scheduler::tp(1000);
task* scheduler::currentTask = NULL;
HANDLE scheduler::threadHandle;
UINT scheduler::threadID;
HANDLE scheduler::eventHandle;

scheduler::
scheduler()
{
	eventHandle = CreateEvent(NULL, FALSE, FALSE, "RMcB-sched");
	threadHandle = (HANDLE) _beginthreadex(NULL, 0, threadMain, NULL, 0, &threadID);
}

scheduler::
~scheduler()
{
}

unsigned WINAPI
scheduler::
threadMain(LPVOID lpvThreadParm)
{
	while (1)
	{
		reorderTasks();
		waitAndHandle();
	}
	return 0; // never reached; we just let thread die when process exits
}

void
scheduler::
reorderTasks()
{
	if (currentTask != NULL) // find out what should be scheduled next
	{
		task* min = (task*) currentTask;
		task* i;
		task* min_prev = NULL;
		task* i_prev = NULL;
		if (currentTask == NULL)
			tw.spr("CurrentTask is NULL\r");
		for (i = (task*) currentTask; i != NULL; i = i->chain)
		{
			if (i == NULL)
				tw.spr("i is NULL\r");
			if (i->td.nwu < min->td.nwu)
			{
				min = i;
				min_prev = i_prev;
			}
			i_prev = i;
		}
		if (min == NULL)
			tw.spr("min is NULL\r");
		if (min_prev)
		{
			min_prev->chain = NULL;
			i_prev->chain = (task*) currentTask;
			currentTask = min;
		}
	}
}

void
scheduler::
waitAndHandle()
{
	DWORD dwTimeout;
	if (currentTask != NULL)
	{
		long delay = currentTask->td.nwu - oap->time(); // calculate time for wakeup of next task
		dwTimeout = ((delay > 0) ? (delay/2) : 0);
	}
	else
		dwTimeout = INFINITE; // no task scheduled; delay forever (until we get MIDI input)
	DWORD result = WaitForSingleObject(eventHandle, dwTimeout);
	if (result == WAIT_TIMEOUT)
		removeAndExecute(); // execute scheduled task
	else
		oap->processQueue(); // process MIDI input
}

void
scheduler::
processQueue()
{
	SetEvent(eventHandle);
}

void
scheduler::
removeAndExecute()
{
	ScheduleTime oaptime = oap->time();
	if (currentTask == NULL || oaptime < currentTask->td.nwu) // this might happen if main thread 
		return;                                // removed a scheduled event while we were sleeping

	task* t = currentTask;
	task_data& td = currentTask->td;
	currentTask = currentTask->chain; // remove task

	if (statistics_on)
		tp.record_lateness( oaptime - td.nwu );
	
	// execute task
	bool retval = false;
	switch (td.ty)
	{
		case task_data::fa_t:
		{
			long r_time = td.fa.func( td.fa.arg );
			if (r_time != NEVER)
			{
				td.nwu += r_time;
				retval = true;
			}
			break;
		}
		case task_data::m_t: oap->write( &(td.m) ); break;
		case task_data::on_t:
			oap->write( &(td.m) );
			td.ty = task_data::m_t;
			td.m.d2 = 0;
			td.nwu += td.m.time;
			retval = true;
			break;
	}

	if (retval == false)
	{
		tp.mon.dec();
		tp.release( t );
	}
	else // implement ZOR
	{
		do_insert(t);
	}
}

err_ind
scheduler::
play_note( const MIDI_msg& on, long dur )
{
	oap->write( &on );
	MIDI_msg off = on;
	off.d2 = 0;
	task_data td( off, oap->time() + dur );
	return insert(td);
}

err_ind
scheduler::
do_insert(task* t)
{
	t->chain = currentTask;
	currentTask = t;
	if (GetCurrentThreadId() != threadID) // this hack is so that if we schedule from another thread,
		SetEvent(eventHandle);             // we will recompute which event is scheduled next
	return no_err;
}

err_ind
scheduler::
cancel(task* t)
{
		if (t == currentTask)
		{
			currentTask = t->chain;
			tp.mon.dec();
			tp.release(t);
			return no_err;
		}
		task* i;
		task* i_prev = NULL;
		// won't hit first time through loop because we know t != currentTask
		for (i = (task*) currentTask; i != NULL; i = i->chain)
		{
			if (i == t)
			{
				i_prev->chain = i->chain; // remove link
				tp.mon.dec();
				tp.release( t );
				return no_err;
			}
			i_prev = i;
		}
		tw.spr("Event not found!\r");
		return err;
}

//--------------------------------------------------------------------------------------------------

// Authors: Ben Denckla, Patrick Pelletier
