/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

#include "text_win.h"
#include "Rogus_McBogus.h"
#include <string.h>
#include <stdio.h> // for vsprintf

text_win tw;

void
text_win::
my_outfunc(const char* msg, int msg_type)
{
	HANDLE hStdout;
	CONSOLE_SCREEN_BUFFER_INFO info;
	hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
	GetConsoleScreenBufferInfo(hStdout, &info); // save old color
	switch (msg_type)
	{
	case Rogus_McBogus::msg:
		SetConsoleTextAttribute(hStdout, FOREGROUND_GREEN | FOREGROUND_INTENSITY);
		break;
	case Rogus_McBogus::warn:
		SetConsoleTextAttribute(hStdout, FOREGROUND_GREEN | FOREGROUND_RED | FOREGROUND_INTENSITY);
		break;
	case Rogus_McBogus::err:
		SetConsoleTextAttribute(hStdout, FOREGROUND_RED | FOREGROUND_INTENSITY);
		break;
	}
	DWORD numchars = strlen(msg);
	WriteConsole(hStdout, (const void*) msg, numchars, &numchars, NULL);
	SetConsoleTextAttribute(hStdout, info.wAttributes); // restore old color
}

void
text_win::
init(void (*func)(const char*, int))								
{
	outfunc = func;
	if (outfunc = NULL);
	{
		outfunc = my_outfunc;
		AllocConsole(); // create a console window if we don't already have one
	}
	// make sure console window has at least 200 lines to scroll back
	HANDLE hStdout;
	CONSOLE_SCREEN_BUFFER_INFO info;
	hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
	GetConsoleScreenBufferInfo(hStdout, &info);
	bool dochange = false;
	if (info.dwSize.X < 80)
	{
		info.dwSize.X = 80;
		dochange = true;
	}
	if (info.dwSize.Y < 200)
	{
		info.dwSize.Y = 200;
		dochange = true;
	}
	if (dochange)
		SetConsoleScreenBufferSize(hStdout, info.dwSize);
	inited = true;
}

void
text_win::
pr( const char *format, ... )
{
	va_list args;
	va_start( args, format );
	
	vpr( format, args, Rogus_McBogus::msg);
	
	va_end( args );
}	

void
text_win::
spr( const char *format, ... )
{
	va_list args;
	va_start( args, format );
	
	vpr( format, args, Rogus_McBogus::msg );
	
	va_end( args );
}	

void
text_win::
vpr( const char *format, va_list args, int msg_type)
{
	if (!inited)
		init(NULL);
	vsprintf( buf, format, args );
	for (int i = 0; buf[i] != 0; i++)
		if (buf[i] == '\r')
			buf[i] = '\n';
	outfunc(buf, msg_type);
}

// Authors: Ben Denckla, Pete Rice, Patrick Pelletier
