/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

#ifndef FILEDATA_H
#define FILEDATA_H

#include <stdio.h>
#include "basic_types.h"
#include "UtilPtr.h"
#include "UtilString.h"
#include "UtilArray.h"
#include "MidiMsg.h"

class Filedata
{
private:
	static uint2 swap2(uint2 n)
	{
		unsigned long test = 0x12345678;
		if (*((unsigned char*)(&test)) == 0x12)
			return n;
		else
			return ((n >> 8) | ((n & 0xff) << 8));
	}

	static uint4 swap4(uint4 n)
	{
		unsigned long test = 0x12345678;
		if (*((unsigned char*)(&test)) == 0x12)
			return n;
		else
			return (swap2((uint2)(n >> 16)) | (swap2((uint2)(n & 0xffff)) << 16));
	}

public:
	typedef UtilArray<MidiMsg> MidiMsgArray;
	typedef UtilPtr<MidiMsgArray> MidiMsgArrayPtr;

private:
#pragma pack(push,1)
	struct trk_hdr { int4 chunk_type, byte_count; };
	struct chk_hdr { int4 chunk_type, length; int2 format, ntrks, division; };
#pragma pack(pop)
	struct track
	{
		trk_hdr hdr;
		MidiMsgArrayPtr m;
		void fixdeltas();
		bool operator==(const track& t) { return ((*m == *(t.m))); }
	};
	typedef UtilArray<track> trackArray;
	typedef UtilPtr<trackArray> trackArrayPtr;

	chk_hdr hdr;
	trackArrayPtr trk;
	
	static int dual_purpose_read(uchar* in_raw, track* t, int byte_count, void (*warnfunc)(const char*));

public:
	Filedata(const char* midi_file_name, void (*warnfunc)(const char*));
	void printToFile(const char* file_name) const;
	MidiMsgArrayPtr getTrack(int i) const { return (*trk)[i].m; }
	int2 getFormat() const { return hdr.format; }
	int2 getNumTracks() const { return hdr.ntrks; }
	int2 getDivision() const { return hdr.division; }
};

#endif
// Authors: Ben Denckla, Patrick Pelletier
