/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

#ifndef PERCHANNEL_H
#define PERCHANNEL_H

#include "basic_types.h"
#include "Rogus.h"
#include "OMS_app.h"

template <class T>
class PerChannel {
private:
	const int numNodes;
	struct channel {
		ioref node;
		T elements[16];
	};
	channel* array;
public:
	PerChannel();
	~PerChannel();
	T& operator()(ioref, int);
	void iterate(void (*func)(ioref, int, T*, void*), void*);
};

template <class T> PerChannel<T>::PerChannel() : numNodes(oap->howManyOutputNodes())
{
	array = new channel[numNodes];
	for (int i = 0; i < numNodes; i++)
		array[i].node = oap->getOutputRefnum(i);
}

template <class T> PerChannel<T>::~PerChannel()
{
	delete[] array;
}

template <class T> T& PerChannel<T>::operator()(ioref n, int c)
{
	int i = 0;
	while (array[i].node != n && i < numNodes-1)
		i++;
	return (array[i].elements[c]);
}

template <class T> void PerChannel<T>::iterate(void (*func)(ioref n, int c, T* e, void* x), void* x)
{
	for (int i = 0; i < numNodes; i++)
		for (int j = 0; j < 16; j++)
			func(array[i].node, j, &(array[i].elements[j]), x);
}

#endif
// Author: Patrick Pelletier
