/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

/* This file gives example of how to create a parser with hooks into your own MIDI input 
processing functions.  In this case, a pitch-bend controlled glissando generator is made. */

#include "gliss_parser.h"
#include "ParserRegistration.h"

int gliss_parser::foo = 0;

static OMS_app *oa;
static int ou_node;
static int last_note = -1;

static void pitch_bend( MIDI_msg *m ) 
{
	int note = m->d2; // pitch bend MSB
	m->set( MIDI_msg::note_on, /*chan*/ 0, note /*d[1](key)*/, 120 /*vel*/, ou_node );
	oa->write(m);
	if ( last_note != -1 )
	{
		m->set( MIDI_msg::note_on, /*chan*/ 0, last_note, 0 /*vel*/, ou_node );
		oa->write(m);
	}
	last_note = note;
}

gliss_parser::
gliss_parser( OMS_app *in_oa )
{
	oa = in_oa;
	ou_node = oa->output_refnum("some_output");
	oa->open_input("some_input");

	msg_hnd[MIDI_msg::pitch_bend] = pitch_bend;
}

 gliss_parser::
~gliss_parser()
{
	MIDI_msg m;
	if ( last_note != -1 )
	{
		m.set( MIDI_msg::note_on, /*chan*/ 0, last_note, 0 /*vel*/, ou_node );
		oa->write(&m);
	}
}

static parser* newFunc()
{
	return (new gliss_parser(oap));
}

static ParserRegistration reg(newFunc, "gliss_parser");

// Author: Ben Denckla
